# Set working directory (folder containing all output files) :
setwd("~/Documents/Oak/Data/AcornDisp_Genotype/Chap1/MEMM/MEMM_Output/FINAL/")


#Libraries :
library("RColorBrewer")
library("ggplot2")
library("plotrix", lib.loc="~/R/i686-pc-linux-gnu-library/3.2")
library(gridExtra)


###########################################################################################################################
################################################### Vérification des paramètres ###########################################
###########################################################################################################################


############################################## MC Markov chains against iterations ########################################



##### Plot : ParamDisp ######
#############################


ParamDisp <- function(x)
{
  ParamDisp_files <- list.files(path = ".",recursive=T,pattern ="ParamDispSeedlings")
  X11(width=10,height=10*1.1)
  par(mfrow = c(2,1))
  
  PARAMDISP <- NULL
  
  
  for (i in 1:length(ParamDisp_files))
  {
    temp <- read.table(ParamDisp_files[i], quote="\"")[-(1:1000),x]
    PARAMDISP <- cbind(PARAMDISP,temp)
  }
  
  nrep=dim(PARAMDISP)[1]
  t=(1:nrep)
  
  plot(0,xlim=c(0,nrep),ylim=c(min(PARAMDISP),max(PARAMDISP)), pch=NA, bty="l",ylab="",xlab="iterations",
       main= paste("Dispersal Parameter #",x,sep=""))
  
  
  colors <- rainbow(n = length(ParamDisp_files)) 
  
  
  for(i in 1:dim(PARAMDISP)[2])
  {
    lines(t, PARAMDISP[,i], col = colors[i])
  }
  
  
  mean_PARAMDISP <- apply(PARAMDISP,1,mean)
  sd_PARAMDISP <- apply(PARAMDISP,1,sd)
  
  plot(0,xlim=c(0,nrep),ylim=c(min(PARAMDISP),max(PARAMDISP)), pch=NA, bty="l",ylab="",xlab="iterations",
       main= paste("Mean of dispersal Parameter #",x,sep=""))
  lines(t, mean_PARAMDISP , lwd = 1.5)
  lines(t, mean_PARAMDISP + sd_PARAMDISP , col = "Grey")
  lines(t, mean_PARAMDISP - sd_PARAMDISP , col = "Grey")
}

# x = Numéro du paramètre (2 < x < 12)
# Exemple : ParamDisp(2)




##### ParamFec ######
#####################

ParamFec <- function(x)
{
  ParamFec_files <- list.files(path = ".",recursive=T,pattern ="ParamFecSeedlings")
  
  X11(width=10,height=10*1.1)
  par(mfrow = c(2,1))
  
  PARAMFEC <- NULL
  for (i in 1:length(ParamFec_files))
  {
    temp <- read.table(ParamFec_files[i], quote="\"")[,x]
    PARAMFEC <- cbind(PARAMFEC,temp)
  }
  
  
  nrep=dim(PARAMFEC)[1]
  t=(1:nrep)
  
  plot(0,xlim=c(0,nrep),ylim=c(min(PARAMFEC),max(PARAMFEC)), pch=NA, bty="l",ylab="",xlab="iterations", 
       main= paste("Fecundity Parameter #",x-1,sep=""))
  
  
  colors <- rainbow(n = length(ParamFec_files))
  
  for(i in 1:dim(PARAMFEC)[2])
  {
    lines(t, PARAMFEC[,i], col = colors[i])
  }
  
  
  mean_PARAMFEC <- apply(PARAMFEC,1,mean)
  sd_PARAMFEC <- apply(PARAMFEC,1,sd)
  
  plot(0,xlim=c(0,nrep),ylim=c(min(PARAMFEC),max(PARAMFEC)), pch=NA, bty="l",ylab="",xlab="iterations",
       main= paste("Mean of Fecundity Parameter #",x,sep=""))
  lines(t, mean_PARAMFEC)
  lines(t, mean_PARAMFEC + sd_PARAMFEC , col = "Grey")
  lines(t, mean_PARAMFEC - sd_PARAMFEC , col = "Grey")
}

  # x = Numéro du paramètre (2 < x < 11)
  # Exemple : ParamFec(2)





################################################## Posterior distributions ###############################################



##### ParamDisp ######
#####################
Dist_ParamDisp <- function(x,nbins=50)
{
  ParamDisp_files <- list.files(path = ".", recursive = TRUE, pattern ="ParamDispSeedlings")
  
  #X11(width=10,height=10*1.1)
  
  PARAMDISP <- NULL
  for (i in 1:length(ParamDisp_files))
  {
    temp <- read.table(ParamDisp_files[i], quote="\"")[-(1:1000),x]
    temp <- cbind(temp,i)
    PARAMDISP <- rbind(PARAMDISP,temp)
  }
  
  PARAMDISP <- as.data.frame(PARAMDISP)
  
  p <- ggplot(PARAMDISP, aes(temp, fill = factor(i))) +
    geom_histogram(alpha = 0.1, aes(), position = 'identity', bins = nbins) +
    xlab(paste("Dispersion Parameter #",x-1,sep="")) +
    theme(legend.position='none')
  #labs(fill = "Replication") 
  
  df <- data.frame(
    a = mean(PARAMDISP$temp),
    b = (max(layer_data(p)$y)+max(layer_data(p)$y)/10),
    c = std.error(as.numeric(tapply(PARAMDISP$temp,PARAMDISP$i,mean)))*2,
    d = max(layer_data(p)$y)+max(layer_data(p)$y)/10
  )
  
  p + geom_tile(data = df, aes(x = a, y = b/2, width = c, height = d), fill = "grey50", alpha = 0.5) +
    geom_segment(data = df, aes(x = a ,xend = a, y = 0, yend = b), colour="red3") +
    theme(legend.position='none')
}
  
  # x = Numéro du paramètre (2 < x < npar+1)
  # Exemple : ParamDisp(2)

grid.arrange(Dist_ParamDisp(2),Dist_ParamDisp(3),Dist_ParamDisp(4),Dist_ParamDisp(5),Dist_ParamDisp(6),
             Dist_ParamDisp(7),Dist_ParamDisp(8),Dist_ParamDisp(9),Dist_ParamDisp(10),Dist_ParamDisp(11),nrow=2,ncol=5)
  # Long calculation but nice...



##### ParamFec ######
#####################
Dist_ParamFec <- function(x,nbins=50)
{
  ParamFec_files <- list.files(path = ".",recursive=T,pattern ="ParamFecSeedlings")
  
  #X11(width=10,height=10*1.1)
  PARAMFEC <- NULL
  for (i in 1:length(ParamFec_files))
  {
    temp <- read.table(ParamFec_files[i], quote="\"")[,x]
    temp <- cbind(temp,i)
    PARAMFEC <- rbind(PARAMFEC,temp)
  }
  
  
  PARAMFEC <- as.data.frame(PARAMFEC)
  
  p <- ggplot(PARAMFEC, aes(temp, fill = factor(i))) + 
    geom_histogram(alpha = 0.1, aes(), position = 'identity', bins = nbins) +
    xlab(paste("Fecundity Parameter #",x-1,sep="")) + 
    theme(legend.position='none')
  #labs(fill = "Replication")
  
  df <- data.frame(
    a = mean(PARAMFEC$temp),
    b = (max(layer_data(p)$y)+max(layer_data(p)$y)/10),
    c = std.error(as.numeric(tapply(PARAMFEC$temp,PARAMFEC$i,mean)))*2,
    d = max(layer_data(p)$y)+max(layer_data(p)$y)/10
  )
  
  p + geom_tile(data = df, aes(x = a, y = b/2, width = c, height = d), fill = "grey50", alpha = 0.5) +
    geom_segment(data = df, aes(x = a, xend = a, y = 0, yend = b), colour="red3") +
    theme(legend.position='none')
  
}

  # x = Numéro du paramètre (2 < x < npar+1)
  # Exemple : ParamFec(2)

grid.arrange(Dist_ParamFec(2),Dist_ParamFec(3),Dist_ParamFec(4),Dist_ParamFec(5),Dist_ParamFec(6),
             Dist_ParamFec(7),Dist_ParamFec(8),Dist_ParamFec(9),Dist_ParamFec(10),nrow=2,ncol=5)
  # Long calculation but nice...


