#////////////////////////////////////////////////////////////////////////////////////////////////
#//                                                                                          ////
#//          MixedEffectsMatingModel v1.0                                                    ////
#//          Copyright - INRA MIA EFPA- 2009                                                ////
#//          Klein EK, Desassis N, Oddou-Muratorio S, Carpentier F                           ////
#//          Licence GPL v3  (http://www.gnu.org/licenses/gpl-3.0.html)                      ////
#//                                                                                          ////
#//          Bayesian estimation of individual fecundities and pollen dispersal kernel       ////
#//          from microsatellite data on seeds and adult trees.                              ////
#//                                                                                          ////
#//          This code was developed by Etienne Klein, INRA Avignon, BioSP                   ////
#//          The reference to this method is                                                 ////
#//          Klein EK, Desassis N, Oddou-Muratorio 2008. Molecular Ecology. 17: 3323-3336    ////
#//                                                                                          ////
#////////////////////////////////////////////////////////////////////////////////////////////////

###### Figure 1 MC Markov chains against iterations #######
###########################################################

figure1= function(file1,file2,ww=10,wcex=1.0){

ParamFec=read.delim(file1,header=FALSE, sep=" ")
ParamDisp=read.delim(file2,header=FALSE, sep=" ")

nrep=dim(ParamFec)[1]
t=(1:nrep)

	X11(width=ww,height=ww*1.1)
	par(mfrow=c(3,2), cex=wcex)

yllik=ParamDisp[,2]
mllik=mean(yllik)
maxllik=max(yllik)
llikmain=paste('A\n','max=',format(maxllik,digits=2),' ; mean=',format(mllik,digits=2))
plot(t/10000, yllik,type='l',main=llikmain, xlab='iterations (x 10000)', ylab='Conditional log likelihood');
lines(c(0,nrep),c(mllik, mllik),col=2,lwd=3)
lines(c(0,nrep),c(maxllik, maxllik),col=4,lwd=3)

yde=ParamFec[,3]
mde=mean(yde)
mmde=median(yde)
de25=sort(yde)[floor(nrep*0.025)]
de95=sort(yde)[ceiling(nrep*0.975)]
demain=paste('B\n','mean=',format(mde,digits=2),' ; median=',format(mmde,digits=2),' ; CI=[',format(de25,digits=2),',',format(de95,digits=2),']')
plot(t/10000, yde,type='l',main=demain, xlab='iterations (x 10000)', ylab='Effective density, dobs/de');
lines(c(0,nrep),c(mde, mde),col=2,lwd=3)
lines(c(0,nrep),c(mde, mmde),col=2,lwd=3)
lines(c(0,nrep),c(de25, de25),col=4,lwd=3)
lines(c(0,nrep),c(de95, de95),col=4,lwd=3)

ydelta=ParamDisp[,3]
mdelta=mean(ydelta)
mmdelta=median(ydelta)
delta25=sort(ydelta)[floor(nrep*0.025)]
delta95=sort(ydelta)[ceiling(nrep*0.975)]
deltamain=paste('C\n','mean=',format(mdelta,digits=2),' ; median=',format(mmdelta,digits=2),' ; CI=[',format(delta25,digits=2),',',format(delta95,digits=2),']')
plot(t/10000, ydelta,type='l',main=deltamain, xlab='iterations (x 10000)', ylab='Mean dispersal distance, delta');
lines(c(0,nrep),c(mdelta, mdelta),col=2,lwd=3)
lines(c(0,nrep),c(mdelta, mmdelta),col=2,lwd=3)
lines(c(0,nrep),c(delta25, delta25),col=4,lwd=3)
lines(c(0,nrep),c(delta95, delta95),col=4,lwd=3)

yb=ParamDisp[,4]
mb=mean(yb)
mmb=median(yb)
b25=sort(yb)[floor(nrep*0.025)]
b95=sort(yb)[ceiling(nrep*0.975)]
bmain=paste('D\n','mean=',format(mb,digits=2),' ; median=',format(mmb,digits=2),' ; CI=[',format(b25,digits=2),',',format(b95,digits=2),']')
plot(t/10000, yb,type='l',main=bmain , xlab='iterations (x 10000)', ylab='Shape parameter, b');
lines(c(0,nrep),c(mb, mb),col=2,lwd=3)
lines(c(0,nrep),c(mb, mmb),col=2,lwd=3)
lines(c(0,nrep),c(b25, b25),col=4,lwd=3)
lines(c(0,nrep),c(b95, b95),col=4,lwd=3)

ym=ParamDisp[,5]
mm=mean(ym)
mmm=median(ym)
m25=sort(ym)[floor(nrep*0.025)]
m95=sort(ym)[ceiling(nrep*0.975)]
mmain=paste('E\n','mean=',format(mm,digits=2),' ; median=',format(mmm,digits=2),' ; CI=[',format(m25,digits=2),',',format(m95,digits=2),']')
plot(t/10000, ym,type='l',main=mmain , xlab='iterations (x 10000)', ylab='Immigration rate, m');
lines(c(0,nrep),c(mm, mm),col=2,lwd=3)
lines(c(0,nrep),c(mm, mmm),col=2,lwd=3)
lines(c(0,nrep),c(m25, m25),col=4,lwd=3)
lines(c(0,nrep),c(m95, m95),col=4,lwd=3)

ys=ParamDisp[,6]
ms=mean(ys)
mms=median(ys)
s25=sort(ys)[floor(nrep*0.025)]
s95=sort(ys)[ceiling(nrep*0.975)]
smain=paste('F\n','mean=',format(ms,digits=2),' ; median=',format(mms,digits=2),' ; CI=[',format(s25,digits=2),',',format(s95,digits=2),']')
plot(t/10000, ys,type='l',main=smain , xlab='iterations (x 10000)', ylab='Selfing rate, m');
lines(c(0,nrep),c(ms, ms),col=2,lwd=3)
lines(c(0,nrep),c(ms, mms),col=2,lwd=3)
lines(c(0,nrep),c(s25, s25),col=4,lwd=3)
lines(c(0,nrep),c(s95, s95),col=4,lwd=3)

}

######     Figure 2 Posterior distributions    #######
######################################################

figure2 = function(file1,file2,file3,npar,demin,demax,dempmin,dempmax,deltamin,deltamax, bmin,bmax, mmin, mmax, smin,smax, ww=10, wcex=1.0){
	
	ParamFec=read.delim(file1,header=FALSE, sep=" ")
	ParamDisp=read.delim(file2,header=FALSE, sep=" ")

	nrep=dim(ParamFec)[1]
	t=(1:nrep)

	IndivFec=read.delim(file3,header=FALSE, sep=" ")[,2:npar+1]
	nrepi=dim(IndivFec)[1]
	MeanIndiv=(apply(IndivFec,1,mean))
	VarIndiv=(apply(IndivFec,1,var))
	ydemp =as.vector(1+VarIndiv/MeanIndiv^2)

	X11(width=ww,height=ww*1.1)
	par(mfrow=c(3,2), cex=wcex)

	xb=demin+(0:50)*(demax-demin)/50
	x= demin+(0:1000)*(demax-demin)/1000
	yde=ParamFec[,3]
	hist(yde,col=8,breaks=c(0,xb,10000),freq=FALSE,xlim=c(demin, demax), main='A', xlab='Theoretical dobs/dep', ylab='PDF'); 
	points(x, 1/x^2/log(x), type='l',lwd=3)
	lines(c(mean(yde),mean(yde)),c(0,100000),col=2,lwd=3)
	lines(c(median(yde),median(yde)),c(0,100000),col=2,lwd=3)

	xb=dempmin+(0:50)*(dempmax-dempmin)/50
	x= dempmin+(0:1000)*(dempmax-dempmin)/1000
	hist(ydemp,col=8,breaks=c(0,xb,10000),freq=FALSE,xlim=c(dempmin, dempmax), main='B', xlab='Empirical dobs/dep', ylab='PDF'); 
	points(x, 1/x^2/log(x), type='l',lwd=3)
	lines(c(mean(ydemp),mean(ydemp)),c(0,100000),col=2,lwd=3)
	lines(c(median(ydemp),median(ydemp)),c(0,100000),col=2,lwd=3)
	
	xb=deltamin+(0:50)*(deltamax-deltamin)/50
	x= deltamin+(0:1000)*(deltamax-deltamin)/1000
	ydelta=ParamDisp[,3]
	hist(ydelta,col=8,breaks=c(0,xb,10000000),freq=FALSE,xlim=c(deltamin, deltamax), main='C', xlab='Mean dispersal distance, delta', ylab='PDF'); 
	points(x, 1/10/x, type='l',lwd=3)
	lines(c(mean(ydelta),mean(ydelta)),c(0,100000),col=2,lwd=3)
	lines(c(median(ydelta),median(ydelta)),c(0,100000),col=2,lwd=3)
	
	xb=bmin+(0:50)*(bmax-bmin)/50
	x=bmin+(0:1000)*(bmax-bmin)/1000
	yb=ParamDisp[,4]
	hist(yb,col=8,breaks=c(0,xb,100000),freq=FALSE,xlim=c(bmin,bmax),main='D', xlab='Shape parameter, b', ylab='PDF'); 
	points(x, 1/x, type='l',lwd=3)
	lines(c(mean(yb),mean(yb)),c(0,100000),col=2,lwd=3)
	lines(c(median(yb),median(yb)),c(0,100000),col=2,lwd=3)

	xb=mmin+(0:50)*(mmax-mmin)/50
	x=mmin+(0:1000)*(mmax-mmin)/1000
	ym=ParamDisp[,5]
	hist(ym,col=8,breaks=c(0,xb,1.1),freq=FALSE,xlim=c(mmin,mmax),main='E', xlab='Immigration rate, m', ylab='PDF'); 
	points(x, 1/x, type='l',lwd=3)
	lines(c(mean(ym),mean(ym)),c(0,100000),col=2,lwd=3)
	lines(c(median(ym),median(ym)),c(0,100000),col=2,lwd=3)

	xb=smin+(0:50)*(smax-smin)/50
	x=smin+(0:1000)*(smax-smin)/1000
	ys=ParamDisp[,6]
	hist(ys,col=8,breaks=c(0,xb,1.1),freq=FALSE,xlim=c(smin,smax),main='F', xlab='Selfing rate, s', ylab='PDF'); 
	points(x, 1/10/x, type='l',lwd=3)
	lines(c(mean(ys),mean(ys)),c(0,100000),col=2,lwd=3)
	lines(c(median(ys),median(ys)),c(0,100000),col=2,lwd=3)

	}
	



######     Figure 3 Individual fecundities     #######
######################################################


figure3=function(file1, npar, covar=8, ww=10, wcex=1.0){
	
	IndivFec=read.delim(file1,header=FALSE, sep=" ")[,2:(npar+1)]
	nrepi=dim(IndivFec)[1]
	
	MeanIndiv=(apply(IndivFec,2,mean))
	MeanIndiv =as.vector(MeanIndiv/mean(MeanIndiv))
	
	q1MCMC=apply(IndivFec,2,function(p){quantile(p,0.975,type=4,na.rm=TRUE)})
	q2MCMC=apply(IndivFec,2,function(p){quantile(p,0.025,type=4,na.rm=TRUE)})
	pourbox=cbind(rep(0,npar),rep(0,npar),rep(0,npar),MeanIndiv-q2MCMC,rep(0,npar))


	X11(width=ww,height=ww*1.1)
	par(mfrow=c(3,2), cex=wcex)

	barplot(MeanIndiv, main='A', xlab='Pollen donors', ylab='Individual fecundity (posterior mean)',names.arg=rep("",npar),col=covar)
	symbols(0.7+(0:(npar-1))*1.2, MeanIndiv,boxplots=pourbox,add=TRUE)
	
	hist(MeanIndiv,breaks=70,xlim=c(0,max(MeanIndiv)),freq=FALSE,main='B', xlab='Individual fecundity (posterior mean)', ylab='PDF',col=8)
	
	return(rbind(MeanIndiv, q1MCMC, q2MCMC))
	
	}

######     Figure 3 Individual fecundities     #######
######################################################

figure4=function(file1, npar, xy, magn=1,covar=8, ww=10, wcex=1.0){

	IndivFec=read.delim(file1,header=FALSE, sep=" ")[,2:(npar+1)]
	nrepi=dim(IndivFec)[1]
	
	MeanIndiv=(apply(IndivFec,2,mean))
	MeanIndiv =as.vector(MeanIndiv/mean(MeanIndiv))

	X11(width=ww,height=ww)
	par(cex=wcex)	

	plot(xy[,1],xy[,2],pch=".",xlab='', ylab='')
	symbols(xy[,1],xy[,2], circles=MeanIndiv,inches=.15*magn,bg=covar,lwd=2,add=TRUE)
	}
