////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                          ////
//          RandomSeedlingModel v0                                                          ////
//          Copyright - INRA - 2009                                                        ////
//          Klein EK, Oddou-Muratorio S                                                     ////
//          Licence GPL                                                                     ////
//                                                                                          ////
//          Bayesian estimation of male and female individual fecundities                   ////
//          and pollen and seed dispersal kernel                                            ////
//          from microsatellite data on seedlings and adult trees.                          ////
//                                                                                          ////
//          This code was developed by Etienne Klein, INRA Avignon, BioSP                   ////
//          The reference to this method is                                                 ////
//                                                                                          ////
//                                                                                          ////
////////////////////////////////////////////////////////////////////////////////////////////////


#include <iostream>
#include <string>
#include <vector>
#include <fstream>
#include <map>

#include "genotype.h"
#include "locus.h"
#include "individu.h"
#include "parent.h"
#include "graine.h"
#include "MEMM_util.h"
#include "MEMM_gamma.h"
#include <math.h>
#include <time.h>
#include <boost/random/variate_generator.hpp>
#include <boost/random/lagged_fibonacci.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/random/lognormal_distribution.hpp>
#include <boost/random/uniform_real.hpp>
#include <boost/math/distributions/gamma.hpp>
#include <boost/math/special_functions/gamma.hpp>

using namespace std;

//struct PerePot { int pere_pot; double pere_prob;};
struct CouplePot { int mere_pot; int pere_pot; double couple_prob;};


///////////////////////////////////////////////////////////////////////////////////////
//          Definition des fonctions de vraisemblance utiles
///////////////////////////////////////////////////////////////////////////////////////

/*
double gammln(double xx){
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,
		24.01409824083091,-1.231739572450155,
		0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;j++) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
	}
*/
long double loglik2 (int ns, int npar, const vector<vector<double> > & distmp, const vector<vector<double> > & distsm,
                const vector <double > & probmig, const vector <int > & nbmeres, const vector <int > & nbcouples,
                const vector< vector < PerePot > > & probmeres, const vector< vector < CouplePot > > & probcouples,
                const vector<double> & fecs, const vector<double> & fecp, double deltas, double bs, double migs,
                double deltap, double bp, double migp, double self) {

                vector < vector<long double> > matp (npar, vector <long double> (npar, 0.) );
                vector < vector<long double> > mats (ns, vector <long double> (npar, 0.) );
                vector <long double> totp (npar);
                vector <long double> tots (ns);
                double as, abs, ap, abp, liktemp=0, pip=0, pip2=0;
                int pbm=0;

    as = deltas*exp(gammln(2/bs)-gammln(3/bs));
    abs=-pow(as,-bs);
    ap = deltap*exp(gammln(2/bp)-gammln(3/bp));
    abp=-pow(ap,-bp);

    for (int m=0; m<npar; m++){
        totp[m]=0;
        for (int p=0; p<m; p++){
            matp[m][p]=exp(abp*pow(distmp[m][p],bp))*fecp[p];
            if (isnan(matp[m][p])){matp[m][p]=0; pbm=1;}
            totp[m]+=matp[m][p];
        }
        for (int p=m+1; p<npar; p++){
            matp[m][p]=exp(abp*pow(distmp[m][p],bp))*fecp[p];
            if (isnan(matp[m][p])){matp[m][p]=0; pbm=1;}
            totp[m]+=matp[m][p];
        }
        matp[m][m]=self*totp[m]/(1-migp-self);
        }

    for (int s=0; s<ns; s++){
        tots[s]=0;
        for (int m=0; m<npar; m++){
            mats[s][m]=exp(abs*pow(distsm[s][m],bs))*fecs[m];
            if (isnan(mats[s][m])){mats[s][m]=0; pbm=1;}
            tots[s]+=mats[s][m];
        }}


    if (pbm==1) { cout << " Warning: Overflow. One component of the dispersal matrix was not defined. " << endl;}
    //for(int p=0; p<npar; p++) {cout << mat[0][p]/tot[0] << " ; "; }
    //cout << endl;

    for (int s=0; s<ns; s++){
        if( probmig[s]>0 && tots[s]>0) {
            pip=0;pip2=0;
            for (int m=0; m<nbmeres[s]; m++) {
                pip += (probmeres[s][m].pere_prob)*mats[s][probmeres[s][m].pere_pot];}
            for (int m=0; m<nbcouples[s]; m++) {
                pip2 += (probcouples[s][m].couple_prob)*mats[s][probcouples[s][m].mere_pot]*matp[probcouples[s][m].mere_pot][probcouples[s][m].pere_pot]/totp[probcouples[s][m].mere_pot];}
            liktemp += log(probmig[s]*migs+(1-migs)/tots[s]*(pip*migp+(1-migp-self)*pip2));
            }}

    return liktemp;
    }

/*
double loglikg (int npar, const vector<double> & fec, double A) {

    // Parametrage de la loi gamma en A, B.
    // A=1/alpha; B=alpha x beta = 1. fixe
    // k=shape=1/A; theta=scale=A
    // Esperance = B; dobs/de = Var/E/E = A

    double liktemp=0, a1, a2, a3, a4;
    a1=1/A; a2=(1-A)/A; a3=log(A)/A; a4=gammln(1/A);
    for (int p=0; p<npar; p++) {
        liktemp += -fec[p]*a1 + log(fec[p])*a2 - a3 - a4;
    }
    return liktemp;
    }
*/

/*
double loglikln (int npar, const vector<double> & fec, double A) {

    // Parametrage de la loi normale en A.
    // A=dobs/de = SIGMA2 + 1 = exp(sigma2);
    // La gaussienne suit une loi d'Esperance = -sigma2/2; de variance sigma2

    double sigma2=log(A);
    double a1=0.5*log(sigma2);
    double liktemp=0;
    for (int p=0; p<npar; p++) {
        liktemp += - a1 - (log(fec[p]) + sigma2/2 )*(log(fec[p]) + sigma2/2 )/2/sigma2;
    }
    return liktemp;
    }
*/

///////////////////////////////////////////////////////////////////////////////////////
//          Ca commence
///////////////////////////////////////////////////////////////////////////////////////


int main()
{
    ifstream Tunes("./ParametersSeedlings.txt");
    string file_in1, file_in2, fileFA, file_inFA;
    int Nl, Nql, Nqt, Nw;
    Tunes >> file_in1 >> file_in2;
    Tunes >> Nl >> Nql >> Nqt >> Nw;
    Tunes >> fileFA;
    if ( fileFA=="file" ) {Tunes >> file_inFA;}
    cout << file_in1 << "..." << file_in2 << "..."<< Nl << "..."<<Nql<< "..."<<Nqt << endl;
    individu nul("nul", genotype(Nl) );
    //nul.afficher();

///////////////////////////////////////////////////////////////////////////////////////
//          Chargement des informations sur les parents potentiels
///////////////////////////////////////////////////////////////////////////////////////

    //file_in1="/Users/klein/Chercher/C/Genotypes1/AlisierPar.txt";
    ifstream parents(file_in1.c_str());
    string name;
    int Npar=0;
    vector<parent *> tousparents;  //le vecteur contenant
    map<string,int>  IDpere;

    cout << endl << endl << "Loading parents...";
    while (parents >> name){
        cout << name << "...";
        tousparents.push_back(new parent(name,parents, Nql, Nqt, Nl));
        Npar++;}

    for (int k=0; k<Npar; k++){
        //(*tousparents[k]).afficher();
        IDpere[(*tousparents[k]).getName()]=k;
        }

    cout << endl << "Loading parents : OK. " << Npar << " parents loaded. " << endl << endl;
    parents.close();

///////////////////////////////////////////////////////////////////////////////////////
//          Chargement des informations sur les descendants
///////////////////////////////////////////////////////////////////////////////////////


    vector<parent *> toussemis;
    ifstream semis(file_in2.c_str());
    int Ns=0;
    string nomtemp;

    cout << "Loading seedlings...";
    while (semis >> name){
        cout << name << "...";
        toussemis.push_back(new parent(name,semis, 0 , 0, Nl));
        Ns++;
        }
    cout << endl << "Loading seedlings : OK. " << Ns << " seedlings loaded. " <<endl << endl;
    semis.close();

///////////////////////////////////////////////////////////////////////////////////////
//              Calculs des distances ...
///////////////////////////////////////////////////////////////////////////////////////


                    vector < vector<double> > DistMP (Npar, vector<double> (Npar, 0.));
                    vector < vector<double> > DistSM (Ns, vector<double> (Npar, 0.));

    for (int m=0; m<Npar; m++){
        for (int p=0; p<Npar; p++){
            DistMP[m][p]=(*tousparents[m]).dist(*tousparents[p]);}}

    for (int s=0; s<Ns; s++){
        for (int m=0; m<Npar; m++){
            DistSM[s][m]=(*toussemis[s]).dist(*tousparents[m]);}}

 //   for (int p=0; p<Npar; p++){cout << DistMP[0][p] << " | ";}
  //  cout<< endl;

///////////////////////////////////////////////////////////////////////////////////////
// Remplissage des frquences allliques et des maps utiles
///////////////////////////////////////////////////////////////////////////////////////

    int ntemp;
    double floattemp, stemp;
    vector< vector<int> > AllSize (Nl);
    vector< vector<double> > AllFreq (Nl);
    vector< map<int,int> > SizeAll (Nl);
    vector< map<int,double> > SizeFreq (Nl);
    vector< int > Na (Nl, 0.);


if (fileFA=="file") {

                    ifstream fall(file_inFA.c_str());

    cout << "Loading allelic frequencies from file..." << endl;

    for (int l=0; l<Nl; l++) {
        fall >> Na[l];
        stemp=0;
        for (int a=0; a<Na[l]; a++) {
            fall >> ntemp >> floattemp;
            stemp+=floattemp;
            AllSize[l].push_back(ntemp);
            AllFreq[l].push_back(floattemp);
            SizeAll[l][ntemp]=a;
        }
    cout << "Locus " << l << " : " << Na[l] << " alleles" <<endl;
        for (int a=0; a<Na[l]; a++) {
            AllFreq[l][a] /= stemp;
            SizeFreq[l][AllSize[l][a]]=AllFreq[l][a];
            cout << AllSize[l][a] << ":" << AllFreq[l][a] << " | ";
        }
    cout << endl;
    };
    cout << "Loading of allelic frequencies : OK. " <<endl << endl;
    fall.close();
}

else {
    cout << "Computing allelic frequencies from adult trees..." << endl;
    for (int l=0; l<Nl; l++) {
         ntemp=0;
         for (int k=0; k<Npar; k++) {
             for (int m=0; m<2; m++) {
                int all = (*tousparents[k]).getGeno(l)._allele[m];
                if (all>-1) {
                    if ( (SizeAll[l].find(all)) == SizeAll[l].end() ) {
                        Na[l]++;ntemp++;
                        //cout << Na[l] << "...";
                        AllSize[l].push_back(all);
                        AllFreq[l].push_back(1.);
                        SizeAll[l][all]=Na[l]-1;
                        }
                    else {
                        ntemp++;
                        AllFreq[l][SizeAll[l][all]]++;
                        }}
    }}
    cout << "Locus " << l << " : " << Na[l] << " alleles (" << ntemp << ")" <<endl;

    for (int a=0; a<Na[l]; a++) {
        AllFreq[l][a] /= ntemp;
        SizeFreq[l][AllSize[l][a]]=AllFreq[l][a];
        cout << AllSize[l][a] << ":" << AllFreq[l][a] << " | ";
    }
    cout << endl;
    };
    cout << "Estimation of allelic frequencies : OK. " <<endl << endl;
}

///////////////////////////////////////////////////////////////////////////////////////
// Remplissage des matrices des probabilites de transitions
///////////////////////////////////////////////////////////////////////////////////////


    vector<int> NbMeres (Ns);
    vector<int> NbCouples (Ns);
    //vector<double> ProbSelf (Ns);
    vector<double> ProbMig (Ns);
    vector<vector< PerePot > > ProbMeres (Ns);
    vector<vector< CouplePot > > ProbCouples (Ns);

    double probtemp;
    PerePot perepottemp;
    CouplePot couplepottemp;

    cout << "Computing the mendelian likelihoods..." << endl;
    for (int s=0; s<Ns; s++){
        NbMeres[s]=0;
        //ProbSelf[s]=(*toutesgraines[s]).mendel((*tousparents[Meres[MerDesc[s]]]),(*tousparents[Meres[MerDesc[s]]]),Nl,Na,SizeAll,AllFreq);
        //if (ProbSelf[s]>0) {cout << "Seed # " << s << " (" << (*toutesgraines[s]).getName() << ") is possibly issued from selfing. Likelihood = " << ProbSelf[s] << endl;}

        ProbMig[s]=(*toussemis[s]).mendel(nul,nul,Nl,Na,SizeAll,AllFreq);
        if (ProbMig[s]==0) {cout << "Seedling # " << s << " (" << (*toussemis[s]).getName() << ") is certainly not issued from outside or incompatible with the mother. " << endl;}

        for (int m=0; m<Npar; m++) {
            probtemp = (*toussemis[s]).mendel((*tousparents[m]),nul,Nl,Na,SizeAll,AllFreq);
           // cout << probtemp;
            if (probtemp>0) {
                NbMeres[s]++;
                perepottemp.pere_pot=m;
                perepottemp.pere_prob=probtemp;
                ProbMeres[s].push_back(perepottemp);}
                }

        for (int m=0; m<NbMeres[s]; m++) {
            for (int p=0; p<NbMeres[s]; p++) {
                probtemp = (*toussemis[s]).mendel((*tousparents[ProbMeres[s][m].pere_pot]),(*tousparents[ProbMeres[s][p].pere_pot]),Nl,Na,SizeAll,AllFreq);
           // cout << probtemp;
                if (probtemp>0) {
                    NbCouples[s]++;
                    couplepottemp.mere_pot=ProbMeres[s][m].pere_pot;
                    couplepottemp.pere_pot=ProbMeres[s][p].pere_pot;
                    couplepottemp.couple_prob=probtemp;
                    ProbCouples[s].push_back(couplepottemp);}
                }}

        //cout << "Seed #" << s << " Probmig =" << ProbMig[s] << "; Probself =" << ProbSelf[s] << "; NbPeres =" << NbPeres[s]<< endl;
    }
    cout << endl << "Computation of the mendelian likelihoods : OK. " <<endl << endl;

    //cout << "Seed #1: " << NbPeres[1] << " , " << MerDesc[1] << " , " << ProbMig[1] << " ," << ProbSelf[1] << " , " << ProbPeres[1][0].pere_pot << " , " << ProbPeres[1][0].pere_prob << endl;

    //sfor (int p=0; p<Npar; p++) {cout << DistMP[MerDesc[1]][p] << " ; ";}


///////////////////////////////////////////////////////////////////////////////////////
//      Modle LOG-NORMAL. C'est parti pour 2 boucles. Burn-in d'abord, storage ensuite
///////////////////////////////////////////////////////////////////////////////////////

        int seed=1;
        string model;
        int Nburn, Nstep, StepFec;
        string file1, file2, file3;

        double mScales, MScales, mShapes, MShapes, mMigs, MMigs;
        double Scales=0.001, Shapes=1.0, Migs=0.4;
        double GamAs, mGamAs, MGamAs;

        double mScalep, MScalep, mShapep, MShapep, mMigp, MMigp, mSelfp, MSelfp;
        double Scalep=0.001, Shapep=1.0, Migp=0.4, Selfp=0.1;
        double GamAp, mGamAp, MGamAp;

        Tunes >> model >> seed;

        Tunes >> GamAs >> mGamAs >> MGamAs;
        Tunes >> Scales >> mScales >> MScales >> Shapes >> mShapes >> MShapes;
        Tunes >> Migs >> mMigs >> MMigs;

        Tunes >> GamAp >> mGamAp >> MGamAp;
        Tunes >> Scalep >> mScalep >> MScalep >> Shapep >> mShapep >> MShapep;
        Tunes >> Migp >> mMigp >> MMigp >> Selfp >> mSelfp >> MSelfp;

        Tunes >> Nburn >> Nstep >> StepFec;
        Tunes >> file1 >> file2 >> file3;
        vector<double> Vec_Fecs (Npar, 1.0 );
        vector<double> Vec_Fecp (Npar, 1.0 );
        vector<double> Poids (Npar, 1.0 );
        Tunes.close();

        double havant, hapres, lavant, lapres;
        double GamA_Prop;
        vector<double> Vec_Fec_Props (Npar, 1.0 );
        vector<double> Vec_Fec_Propp (Npar, 1.0 );
        double Scale_Prop=0.001, Shape_Prop=1.0, Mig_Prop=0.4, Self_Prop=0.1;
        long double a, ab;

        ofstream ParamFec(file1.c_str());
        ofstream IndivFec(file2.c_str());
        ofstream ParamDisp(file3.c_str());

        boost::normal_distribution<> dist_norm(0, 1.);
        boost::uniform_real<> unif(0,1.0);
        boost::lagged_fibonacci19937 generator(static_cast<long unsigned int>(seed));
        boost::variate_generator<boost::lagged_fibonacci19937&, boost::normal_distribution<> > gauss(generator, dist_norm);
        boost::variate_generator<boost::lagged_fibonacci19937&, boost::uniform_real<> > accept(generator, unif);

        vector<double> covarqtemp(Nqt, 1.0);

        cout << "Poids des individus : ";
        for (int p=0; p<Npar; p++) {
            covarqtemp=(*tousparents[p])._covarquanti;
            for (int k=0; k<Nw; k++){
            Poids[p]*=covarqtemp[k];}
            cout << Poids[p] << " | ";
        }
        cout << endl << endl;


    for (int k=0; k<Npar; k++){
        //(*tousparents[k]).afficher();
        IDpere[(*tousparents[k]).getName()]=k;
        }

//cout << loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, 743.0, 0.33, 0.4, 0.05) << endl;
//for (int k=0; k<Npar; k++) {Vec_Fecp[k]=k;}
//cout << loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fecp, 743.0, 0.33, 0.4, 0.05) << endl;

if (model=="LN") {

//                    double sigma, mu;
//
//    cout << "Burn in iterations ..." <<endl;
//    for (int rep=0; rep<Nburn; rep++) {
//
//        havant = loglikln(Npar, Vec_Fec, GamA);
//        GamAp = pow(GamA, exp(0.2*gauss()));
//        hapres = loglikln(Npar, Vec_Fec, GamAp);
//        if ((GamAp>mGamA) && (GamAp<MGamA) && (accept()<exp(hapres-havant))) {
//            GamA=GamAp;}
//
//        //for (int p=0; p<Npar; p++) {Vec_Fecp[p]=GamA*boost::quantile(boost::gamma_distribution(0.5,0.5);cout << Vec_Fecp[p]<< "...";}
//        lavant = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scale, Shape, Mig, Self);
//        sigma=pow(log(GamA),0.5);
//        mu=-log(GamA)/2;
//        Vec_Fecp=Vec_Fec;
//        for (int p=0; p<Npar; p++) {
//            Vec_Fecp[p]=exp(sigma*gauss()+mu);
//            lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fecp, Scale, Shape, Mig, Self);
//            if (accept()<exp(lapres-lavant)) {
//                Vec_Fec[p]=Vec_Fecp[p]; lavant=lapres;}
//            else {Vec_Fecp[p]=Vec_Fec[p];}
//            }
//
//        Scalep = Scale*exp(0.2*gauss());
//        lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scalep, Shape, Mig, Self);;
//        if ((Scalep>mScale) && (Scalep<MScale) && (accept()<exp(lapres-lavant))) {
//            Scale=Scalep;
//            lavant=lapres;}
//        Shapep = Shape*exp(0.2*gauss());
//        lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scale, Shapep, Mig, Self);;
//        if ((Shapep>mShape) && (Shapep<MShape) && (accept()<exp(lapres-lavant))) {
//            Shape=Shapep;
//            lavant=lapres;}
//        Migp = Mig*exp(0.2*gauss());
//        lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scale, Shape, Migp, Self);;
//        if ((Migp>mMig) && (Migp<MMig) && (accept()<exp(lapres-lavant))) {
//            Mig=Migp;
//            lavant=lapres;}
//        Selfp = Self*exp(0.2*gauss());
//        lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scale, Shape, Mig, Selfp);;
//        if ((Selfp>mSelf) && (Selfp<MSelf) && (accept()<exp(lapres-lavant))) {
//            Self=Selfp;
//            lavant=lapres;}
//    };
//    cout << "Burn in iterations : OK" << endl <<endl;
//
//    cout << "Final iterations ";
//
//    cout << "..."<<endl;
//
//    for (int rep=0; rep<Nstep; rep++) {
//
//        if (rep % 1000 == 0) {cout << rep << "..."<< endl;}
//        havant = loglikln(Npar, Vec_Fec, GamA);
//        GamAp = pow(GamA, exp(0.2*gauss()));
//        hapres = loglikln(Npar, Vec_Fec, GamAp);
//        //cout << GamA << ":" << havant<< "  |  ";
//        //cout << GamAp << ":" << hapres << endl;
//        if ((GamAp>mGamA) && (GamAp<MGamA) && (accept()<exp(hapres-havant))) {
//            GamA=GamAp;havant=hapres;}
//        ParamFec << rep << " " << havant << " " << GamA << endl;
//
//        //for (int p=0; p<Npar; p++) {Vec_Fecp[p]=GamA*boost::quantile(boost::gamma_distribution(0.5,0.5);cout << Vec_Fecp[p]<< "...";}
//        lavant = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scale, Shape, Mig, Self);
//        sigma=pow(log(GamA),0.5);
//        mu=-log(GamA)/2;
//        Vec_Fecp=Vec_Fec;
//        for (int p=0; p<Npar; p++) {
//            Vec_Fecp[p]=exp(pow(log(GamA),0.5)*gauss()+mu);
//            lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fecp, Scale, Shape, Mig, Self);
//            if (accept()<exp(lapres-lavant)) {
//                Vec_Fec[p]=Vec_Fecp[p]; lavant=lapres;}
//            else {Vec_Fecp[p]=Vec_Fec[p];}
//            //IndivFec << Vec_Fec[p] << " ";
//            }
//
//        if (rep % StepFec == 0) {
//            IndivFec << rep << " " ;
//            for (int p=0; p<Npar; p++) {IndivFec << Vec_Fec[p] << " ";}
//            IndivFec << endl;
//        }
//
//        Scalep = Scale*exp(0.2*gauss());
//        lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scalep, Shape, Mig, Self);
//        if ((Scalep>mScale) && (Scalep<MScale) && (accept()<exp(lapres-lavant))) {
//            Scale=Scalep;
//            lavant=lapres;}
//        Shapep = Shape*exp(0.2*gauss());
//        lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scale, Shapep, Mig, Self);
//        if ((Shapep>mShape) && (Shapep<MShape) && (accept()<exp(lapres-lavant))) {
//            Shape=Shapep;
//            lavant=lapres;}
//        a = Scale*exp(gammln(2/Shape)-gammln(3/Shape));
//        ab=-pow(a,-Shape);
//        Migp = Mig*exp(0.2*gauss());
//        lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scale, Shape, Migp, Self);
//        if ((Migp>mMig) && (Migp<MMig) && (accept()<exp(lapres-lavant))) {
//            Mig=Migp;
//            lavant=lapres;}
//        Selfp = Self*exp(0.2*gauss());
//        lapres = loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, Vec_Fec, Scale, Shape, Mig, Selfp);
//        if ((Selfp>mSelf) && (Selfp<MSelf) && (accept()<exp(lapres-lavant))) {
//            Self=Selfp;
//            lavant=lapres;}
//        ParamDisp << rep << " " << lavant << " " << Scale << " " << Shape << " " << Mig << " " << Self << " " << a << " " << ab << endl;
//
//    }

    cout << endl << "MCMC finished. You can now visualize the results from R" << endl << endl;
    ParamFec.close();
    IndivFec.close();
    ParamDisp.close();
}
else {

    boost::math::gamma_distribution<> gammas(1./GamAs,GamAs);
    boost::math::gamma_distribution<> gammap(1./GamAp,GamAp);

    MEMM_loi * pLoi = new MEMM_gamma(GamAs,accept);
    // TODO Ajouter une deuxime loi pour GamAp

    cout << "Burn in iterations ..." <<endl;
    for (int rep=0; rep<Nburn; rep++) {

        havant = pLoi->logLik(Npar, Vec_Fecs, GamAs);
        GamA_Prop = pow(GamAs, exp(0.2*gauss()));
        hapres = pLoi->logLik(Npar, Vec_Fecs, GamA_Prop);
        if ((GamA_Prop>mGamAs) && (GamA_Prop<MGamAs) && (accept()<exp(hapres-havant))) {
            GamAs=GamA_Prop;}

        havant = pLoi->logLik(Npar, Vec_Fecp, GamAp);
        GamA_Prop = pow(GamAp, exp(0.2*gauss()));
        hapres = pLoi->logLik(Npar, Vec_Fecp, GamA_Prop);
        if ((GamA_Prop>mGamAp) && (GamA_Prop<MGamAp) && (accept()<exp(hapres-havant))) {
            GamAp=GamA_Prop;}

        lavant = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scalep, Shapep, Migp, Selfp);
        Vec_Fec_Props=Vec_Fecs;
        Vec_Fec_Propp=Vec_Fecp;
        gammas = boost::math::gamma_distribution<>(1./GamAs,GamAs);
        gammap = boost::math::gamma_distribution<>(1./GamAp,GamAp);
        for (int p=0; p<Npar; p++) {
            Vec_Fec_Props[p]=quantile(gammas,accept())*Poids[p];
            Vec_Fec_Propp[p]=quantile(gammap,accept())*Poids[p];
            lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fec_Props, Vec_Fec_Propp, Scales, Shapes, Migs, Scalep, Shapep, Migp, Selfp);
            if (accept()<exp(lapres-lavant)) {
                Vec_Fecs[p]=Vec_Fec_Props[p]; Vec_Fecp[p]=Vec_Fec_Propp[p]; lavant=lapres;}
            else {Vec_Fec_Props[p]=Vec_Fecs[p];Vec_Fec_Propp[p]=Vec_Fecp[p];}
            }

        Scale_Prop = Scales*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scale_Prop, Shapes, Migs, Scalep, Shapep, Migp, Selfp);
        if ((Scale_Prop>mScales) && (Scale_Prop<MScales) && (accept()<exp(lapres-lavant))) {
            Scales=Scale_Prop;
            lavant=lapres;}
        Shape_Prop = Shapes*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shape_Prop, Migs, Scalep, Shapep, Migp, Selfp);
        if ((Shape_Prop>mShapes) && (Shape_Prop<MShapes) && (accept()<exp(lapres-lavant))) {
            Shapes=Shape_Prop;
            lavant=lapres;}
        Mig_Prop = Migs*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Mig_Prop, Scalep, Shapep, Migp, Selfp);
        if ((Mig_Prop>mMigs) && (Mig_Prop<MMigs) && (accept()<exp(lapres-lavant))) {
            Migs=Mig_Prop;
            lavant=lapres;}

        Scale_Prop = Scalep*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scale_Prop, Shapep, Migp, Selfp);
        if ((Scale_Prop>mScalep) && (Scale_Prop<MScalep) && (accept()<exp(lapres-lavant))) {
            Scalep=Scale_Prop;
            lavant=lapres;}
        Shape_Prop = Shapep*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scalep, Shape_Prop, Migp, Selfp);
        if ((Shape_Prop>mShapep) && (Shape_Prop<MShapep) && (accept()<exp(lapres-lavant))) {
            Shapep=Shape_Prop;
            lavant=lapres;}
        Mig_Prop = Migp*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scalep, Shapep, Mig_Prop, Selfp);
        if ((Mig_Prop>mMigp) && (Mig_Prop<MMigp) && (accept()<exp(lapres-lavant))) {
            Migp=Mig_Prop;
            lavant=lapres;}
        Self_Prop = Selfp*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scalep, Shapep, Migp, Self_Prop);
        if ((Selfp>mSelfp) && (Selfp<MSelfp) && (accept()<exp(lapres-lavant))) {
            Selfp=Self_Prop;
            lavant=lapres;}
    };
    cout << "Burn in iterations : OK" << Nburn<< endl <<endl;

    cout << "Final iterations "<<Nstep;

    cout << "..."<<endl;

    for (int rep=0; rep<Nstep; rep++) {

        if (rep % 1000 == 0) {cout << rep << "..."<< endl;}

        havant = pLoi->logLik(Npar, Vec_Fecs, GamAs);
        GamA_Prop = pow(GamAs, exp(0.2*gauss()));
        hapres = pLoi->logLik(Npar, Vec_Fecs, GamA_Prop);
        if ((GamA_Prop>mGamAs) && (GamA_Prop<MGamAs) && (accept()<exp(hapres-havant))) {
            GamAs=GamA_Prop;}
        ParamFec << rep << " " << hapres << " " << GamAs << " ";

        havant = pLoi->logLik(Npar, Vec_Fecp, GamAp);
        GamA_Prop = pow(GamAp, exp(0.2*gauss()));
        hapres = pLoi->logLik(Npar, Vec_Fecp, GamA_Prop);
        if ((GamA_Prop>mGamAp) && (GamA_Prop<MGamAp) && (accept()<exp(hapres-havant))) {
            GamAp=GamA_Prop;}
        ParamFec << hapres << " " << GamAp << endl;

        lavant = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scalep, Shapep, Migp, Selfp);
        Vec_Fec_Props=Vec_Fecs;
        Vec_Fec_Propp=Vec_Fecp;
        gammas = boost::math::gamma_distribution<>(1./GamAs,GamAs);
        gammap = boost::math::gamma_distribution<>(1./GamAp,GamAp);
        for (int p=0; p<Npar; p++) {
            Vec_Fec_Props[p]=quantile(gammas,accept());
            Vec_Fec_Propp[p]=quantile(gammap,accept());
            lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fec_Props, Vec_Fec_Propp, Scales, Shapes, Migs, Scalep, Shapep, Migp, Selfp);
            if (accept()<exp(lapres-lavant)) {
                Vec_Fecs[p]=Vec_Fec_Props[p]; Vec_Fecp[p]=Vec_Fec_Propp[p]; lavant=lapres;}
            else {Vec_Fec_Props[p]=Vec_Fecs[p];Vec_Fec_Propp[p]=Vec_Fecp[p];}
            }

        if (rep % StepFec == 0) {
            IndivFec << rep << " " ;
            for (int p=0; p<Npar; p++) {IndivFec << Vec_Fecs[p] << " ";}
            for (int p=0; p<Npar; p++) {IndivFec << Vec_Fecp[p] << " ";}
            IndivFec << endl;
        }

        Scale_Prop = Scales*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scale_Prop, Shapes, Migs, Scalep, Shapep, Migp, Selfp);
        if ((Scale_Prop>mScales) && (Scale_Prop<MScales) && (accept()<exp(lapres-lavant)*Scale_Prop*Scales)) {
            Scales=Scale_Prop;
            lavant=lapres;}
        Shape_Prop = Shapes*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shape_Prop, Migs, Scalep, Shapep, Migp, Selfp);
        if ((Shape_Prop>mShapes) && (Shape_Prop<MShapes) && (accept()<exp(lapres-lavant)*Shape_Prop/Shapes)) {
            Shapes=Shape_Prop;
            lavant=lapres;}
        Mig_Prop = Migs*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Mig_Prop, Scalep, Shapep, Migp, Selfp);
        if ((Mig_Prop>mMigs) && (Mig_Prop<MMigs) && (accept()<exp(lapres-lavant))) {
            Migs=Mig_Prop;
            lavant=lapres;}

        Scale_Prop = Scalep*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scale_Prop, Shapep, Migp, Selfp);
        if ((Scale_Prop>mScalep) && (Scale_Prop<MScalep) && (accept()<exp(lapres-lavant)*Scale_Prop/Scalep)) {
            Scalep=Scale_Prop;
            lavant=lapres;}
        Shape_Prop = Shapep*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scalep, Shape_Prop, Migp, Selfp);
        if ((Shape_Prop>mShapep) && (Shape_Prop<MShapep) && (accept()<exp(lapres-lavant)*Shape_Prop/Shapep)) {
            Shapep=Shape_Prop;
            lavant=lapres;}
        Mig_Prop = Migp*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scalep, Shapep, Mig_Prop, Selfp);
        if ((Mig_Prop>mMigp) && (Mig_Prop<MMigp) && (accept()<exp(lapres-lavant))) {
            Migp=Mig_Prop;
            lavant=lapres;}
        Self_Prop = Selfp*exp(0.2*gauss());
        lapres = loglik2(Ns, Npar, DistMP, DistSM, ProbMig, NbMeres, NbCouples, ProbMeres, ProbCouples, Vec_Fecs, Vec_Fecp, Scales, Shapes, Migs, Scalep, Shapep, Migp, Self_Prop);
        if ((Self_Prop>mSelfp) && (Self_Prop<MSelfp) && (accept()<exp(lapres-lavant))) {
            Selfp=Self_Prop;
            lavant=lapres;}

        ParamDisp << rep << " " << lavant << " " << Scales << " " << Shapes << " " << Migs << " " << Scalep << " " << Shapep << " " << Migp << " " << Selfp << endl;
    }

    cout << endl << "MCMC finished. You can now visualize the results from R" << endl << endl;
    ParamFec.close();
    IndivFec.close();
    ParamDisp.close();



};

///////////////////////////////////////////////////////////////////////////////////////
//      Remplissage de la matrice des probabilits de transitions
///////////////////////////////////////////////////////////////////////////////////////



//    time_t seconds = time (NULL);
//    double chronotemp;
//
//    cout << seconds << " | 10000 calculs de loglik1 | ";
//    vector<double> vec_init (Npar, 1.0f );
//    //cout << loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, vec_init, 100, 0.75, 0.40, 0.05);
//    for (int rep=0; rep <10000; rep++) {chronotemp=loglik1(Nm, Ns, Npar, DistMP, Meres, MerDesc, ProbMig, ProbSelf, NbPeres, ProbPeres, vec_init, 100, 0.75, 0.40, 0.05);}
//    seconds = time (NULL);
//    cout << seconds << endl;




///////////////////////////////////////////////////////////////////////////////////////
//      Nettoyage final
///////////////////////////////////////////////////////////////////////////////////////

    for (int k=0; k<Npar; k++) {delete tousparents[k];}
    for (int k=0; k<Ns; k++) {delete toussemis[k];}

    return 0;
}

